"""
Define a base class and prepare ZopeTestCase to be used in
StagingAddOn tests

Based on RichDocument tests.

$Id: utils.py 25289 2006-06-23 08:29:41Z dbaty $
"""

from Products.StagingAddOn import config
config.TESTING = True

## Import the base test case classes
from Testing import ZopeTestCase
from Products.PloneTestCase import PloneTestCase

## Make ZopeTestCase aware of the standard products

## These install (or fail) quietly
ZopeTestCase.installProduct('CMFCore', quiet=1)
ZopeTestCase.installProduct('CMFDefault', quiet=1)
ZopeTestCase.installProduct('CMFCalendar', quiet=1)
ZopeTestCase.installProduct('CMFTopic', quiet=1)
ZopeTestCase.installProduct('DCWorkflow', quiet=1)
ZopeTestCase.installProduct('CMFHelpIcons', quiet=1)
ZopeTestCase.installProduct('CMFQuickInstallerTool', quiet=1)
ZopeTestCase.installProduct('CMFFormController', quiet=1)
ZopeTestCase.installProduct('GroupUserFolder', quiet=1)
ZopeTestCase.installProduct('ZCTextIndex', quiet=1)
ZopeTestCase.installProduct('TextIndexNG2', quiet=1)
ZopeTestCase.installProduct('SecureMailHost', quiet=1)
ZopeTestCase.installProduct('CMFPlone')
ZopeTestCase.installProduct('Archetypes')
ZopeTestCase.installProduct('PortalTransforms', quiet=1)
ZopeTestCase.installProduct('MimetypesRegistry', quiet=1)
ZopeTestCase.installProduct('kupu', quiet=1)

## These must install cleanly
ZopeTestCase.installProduct('StagingAddOn')

## Set up a Plone site
PRODUCTS = ('StagingAddOn', )
PloneTestCase.setupPloneSite(products=PRODUCTS)


## Import statements
from Products.CMFCore.utils import getToolByName
from Products.StagingAddOn.content.staging_tool import ID as STOOL_ID


class StagingAddOnTestCase(PloneTestCase.PloneTestCase):
    """
    Base class for StagingAddOn tests
    """
    def createTestUsersAndContent(self):
        """
        Create some test users and content

        Four users are created:
        - a manager: 'manager'
        - a reviewer: 'reviewer'
        - two members: 'member1' and 'member2'

        The following items are created:
        + folder
        |--+-- document1
        |--+-- subfolder
        |-----+-- document2
        |-----+-- document3
        + Members
        |--+-- member1
        |-----+-- document
        """
        portal = self.portal

        ## Create users
        mtool = getToolByName(portal, 'portal_membership')
        users = ({'id': 'member1', 'roles': ('Member', )},
                 {'id': 'member2', 'roles': ('Member', )},
                 {'id': 'reviewer', 'roles': ('Reviewer', )},
                 {'id': 'manager', 'roles': ('Manager', )})
        for user in users:
            mtool.addMember(user['id'], user['id'], user['roles'], [],
                            properties={})

        ## Create content
        self.login('manager')
        portal.invokeFactory('Folder', 'folder')
        folder = portal.folder
        folder.invokeFactory('Document', 'document1')
        folder.invokeFactory('Folder', 'subfolder')
        subfolder = folder.subfolder
        subfolder.invokeFactory('Document', 'document2')
        subfolder.invokeFactory('Document', 'document3')
        portal.portal_membership.createMemberArea('member1')
        self.login('member1')
        portal.Members.member1.invokeFactory('Document', 'document')

        self.logout()
